@@ -27,6 +27,10 @@ LLsurface.secr <- function (object, betapar = c('g0', 'sigma'), xval = NULL, yva
                 if (any(names(centre) != object$betanames))
                     stop ("names of 'centre' do not match 'object$betanames'")
         }
+        if (object$detectfn %in% 14:18 & "g0" %in% betapar) {
+            betapar[betapar=="g0"] <- "lambda0"
+            warning ("substituting lambda0 for g0 to match detectfn")
+        }
         betaindices <- match(betapar, names(centre))
         if ((length(betapar) != 2) | (any(is.na(betaindices))))
             stop ("requires two named beta parameters")
