@@ -116,7 +116,7 @@ make.poly <- function (polylist=NULL, x=c(-50,-50,50,50), y=c(-50,50,50,-50),
 {
     makepart <- function (vert) {
         vert <- data.frame(vert)
-        if ((ncol(vert)==2) && !all(c('x','y') %in% names(vert)))
+        if ((ncol(vert)==2) & !all(c('x','y') %in% names(vert)))
             names(vert) <- c('x','y')
         if (any(tail(vert,1) != vert[1,]))   ## close polygon
             vert <- rbind(vert, vert[1,])
@@ -170,7 +170,7 @@ make.transect <- function (transectlist=NULL, x=c(-50,-50,50,50), y=c(-50,50,50,
 {
     makepart <- function (vert) {
         vert <- data.frame(vert)
-        if ((ncol(vert)==2) && !all(c('x','y') %in% names(vert)))
+        if ((ncol(vert)==2) & !all(c('x','y') %in% names(vert)))
             names(vert) <- c('x','y')
         vert
     }
