@@ -16,15 +16,17 @@ write.captures <- function (object, file='', deblank = TRUE, header = TRUE,
         stop ("requires a 'capthist' object")
 
     if (ms(object)) {
-        write.captures (object[[1]], file = file, deblank = deblank,
+        out <- vector('list')
+        out[[1]] <- write.captures (object[[1]], file = file, deblank = deblank,
             header = deparse(substitute(object), control=NULL), append = append,
             sess = session(object)[1], ndec = ndec, covariates = covariates,
                         tonumeric = tonumeric, ...)
         for (i in 2:length(object)) {
-            write.captures (object[[i]], file = file, deblank = deblank,
+            out[[i]] <- write.captures (object[[i]], file = file, deblank = deblank,
                 header = FALSE, append = TRUE, sess = session(object)[i], ndec = ndec,
                 covariates = covariates, tonumeric = tonumeric, ...)
         }
+        invisible(out)
     }
     else {
         n <- nrow(object)
@@ -70,10 +72,6 @@ write.captures <- function (object, file='', deblank = TRUE, header = TRUE,
             else
                 covlist <- names(covs)
             if (length(covlist)>0) {
-##                for (i in 1:length(covlist))
-##                covs[,i] <- as.numeric(covs[,i])
-##                temp <- cbind(temp, covs[ID, covlist, drop=FALSE])
-## 2014-04-05
                 if (tonumeric) {
                     for (i in 1:length(covlist))
                         covs[,i] <- as.numeric(covs[,i])
@@ -96,8 +94,6 @@ write.captures <- function (object, file='', deblank = TRUE, header = TRUE,
         }
         write.table(temp, file = file, row.names = FALSE, col.names = FALSE,
             append = append, quote = FALSE, ...)
-
     }
 }
 ############################################################################################
-
