@@ -4,21 +4,25 @@
     2012-02-01 modified for single mufn
     2014-08-27 mufn moved here from utils.c
     2015-12-21 expanded documentation in header
+    2017-03-21 rename hfn to gfnr, and more consistent naming elsewhere
 
 Three forms are provided:
-hfn  -- distance is an argument (r)
+gfnr -- distance is an argument (r)
 gfn  -- distance is computed from trap amd mask indices and coordinate objects
 gfnL -- distance is computed from trap amd mask indices and lookup table
 
-The function pfn() selects hfn on the fly.
-pfn() is used in pdotpoint, naiveRPSV, simdetect, trappingsingle, trappingmulti,
+ The function pfn() selects gfnr on the fly.
+ pfn() is used in pdotpoint, naiveRPSV, simdetect, trappingsingle, trappingmulti,
 trappingproximity, trappingcount, trappingtimes
 
-All functions return the detection probability g(x). For count detectors this
+ The function zfn() selects zfnr on the fly.
+ zfn() is used in hdotpoint
+ 
+All g--- functions return the detection probability g(x). For count detectors this
 must be transformed to the cumulative hazard -log(1-g(x)) (see function 'hazard' 
 in utils.c).
 
-The hfn and gfn forms are used 
+The gfnr and gfn forms (without distance lookup) are used 
 
 (i) for polygon and transect detectors, 
 (ii) for trappingXXX simulations  (via pfn)
@@ -26,7 +30,7 @@ The hfn and gfn forms are used
 for which the function must be calculated on the fly within the integration algorithm 
 rather than from a lookup precomputed for a fixed set of trap-mask distances.
 
-[gfn is passed to all prwi functions in secr.c, but used only in 
+[zfn is passed to all prwi functions in secr.c, but used only in 
 prwipolygon, prwipolygonX, prwitransect, prwitransectX]
 
 Proposed changes 2015-12-21:
@@ -34,30 +38,32 @@ Proposed changes 2015-12-21:
 -- drop all gfn (but beware of flow-on issues for polygons)
 
 For each form there is a corresponding selection function:
-gethfn
+getzfnr
 getgfn
+getzfn
+getgfnr
 getgfnL
-                                   gethfn    getgfn    getgfnL
-
-fn 0  halfnormal                   hn        ghn       ghnL
-fn 1  hazard-rate                  hr        ghr       ghrL
-fn 2  exponential                  he        ghe       gheL
-fn 3  compound halfnormal          hn        ghnc      ghncL
-fn 4  uniform                      un        gun       gunL
-fn 5  w-exponential                hf        ghf       ghfL 
-fn 6  annular normal               hann      gan       ganL
-fn 7  cumulative lognormal         hcln      gcln      gclnL
-fn 8  cumulative gamma             hcg       gcg       gcgL
-fn 9  binary signal strength       hsigbin   gsigbin   gsigbinL
-fn 10 signal strength              hsig      gsig      gsigL
-fn 11 signal strength + ss         hsigsph   gsigsph   gsigsphL 
-fn 12 signal strength + noise      --        gsigSN    gsigSNL
-fn 13 signal strength + ss + noise --        gsigsphSN gsigsphSNL
-fn 14 hazard halfnormal            hhn       lhn       lhnL
-fn 15 hazard hazard rate           hhr       lhr       lhrL
-fn 16 hazard exponential           hex       lex       lexL
-fn 17 hazard annular normal        han       lan       lanL
-fn 18 hazard cumulative gamma      hcumg     lcg       lcgL
+                                   getzfn    getzfnr    getgfn    getgfnr    getgfnL
+                                             [unused]
+fn 0  halfnormal                   zhn        zhnr        ghn       ghnr       ghnL
+fn 1  hazard-rate                  zhr        zhrr        ghr       ghrr       ghrL
+fn 2  exponential                  zex        zexr        gex       gexr       gexL
+fn 3  compound halfnormal          zhnc       zhncr       ghnc      ghncr      ghncL
+fn 4  uniform                      zun        zunr        gun       gunr       gunL
+fn 5  w-exponential                zhf        zhfr        ghf       ghfr       ghfL
+fn 6  annular normal               zhan       zhanr       gan       ganr       ganL
+fn 7  cumulative lognormal         zcln       zclnr       gcln      gclnr      gclnL
+fn 8  cumulative gamma             zcg        zcgr        gcg       gcgr       gcgL
+fn 9  binary signal strength       zhsigbin   zhsigbinr   gsigbin   gsigbinr   gsigbinL
+fn 10 signal strength              zhsig      zhsigr      gsig      gsigr      gsigL
+fn 11 signal strength + ss         zhsigsph   zhsigsphr   gsigsph   gsigsphr   gsigsphL 
+fn 12 signal strength + noise      --         --          gsigSN    gsigSNr    gsigSNL
+fn 13 signal strength + ss + noise --         --          gsigsphSN gsigsphSNr gsigsphSNL
+fn 14 hazard halfnormal            zhhn       zhhnr       ghhn      ghhnr      ghhnL
+fn 15 hazard hazard rate           zhhr       zhhrr       ghhr      ghhrr      ghhrL
+fn 16 hazard exponential           zhex       zhexr       ghex      ghexr      ghexL
+fn 17 hazard annular normal        zhan       zhanr       ghan      ghanr      ghanL
+fn 18 hazard cumulative gamma      zhcg       zhcgr       ghcg      ghcgr      ghcgL
 
 */
 
@@ -81,31 +87,58 @@ int par3 (int fn) {
 
 /*--------------------------------------------------------------------*/
 double pfn (
-    int fn,
-    double d2val,
-    double g0,
-    double sigma,
-    double z,
-    double miscparm [],
-    double w2)
+        int fn,
+        double d2val,
+        double g0,
+        double sigma,
+        double z,
+        double miscparm [],
+                        double w2)
 {
     double p = -1;
-    fnptr hfn;
+    fnptr gfnr;
     double tmp[4];
-
+    
     if (d2val > w2) 
         p = 0;
     else {
-        hfn = gethfn (fn);
+        gfnr = getgfnr (fn);
         tmp[0] = g0;
         tmp[1] = sigma;
         tmp[2] = z;
         tmp[3] = miscparm[0];
-        p = hfn (tmp, sqrt(d2val));
+        p = gfnr (tmp, sqrt(d2val));
     }
     return (p);
 }
 
+double zfn (
+        int fn,
+        double d2val,
+        double lambda0,
+        double sigma,
+        double z,
+        double miscparm [],
+        double w2)
+{
+    double h = -1;
+    fnptr zfnr;
+    double tmp[4];
+    
+    if (d2val > w2) 
+        h = 0;
+    else {
+        zfnr = getzfnr (fn);
+        tmp[0] = lambda0;
+        tmp[1] = sigma;
+        tmp[2] = z;
+        tmp[3] = miscparm[0];
+        h = zfnr (tmp, sqrt(d2val));
+    }
+    return (h);
+}
+
+/*
 fnptr gethfn (int fn) 
 {
     if (fn == 0)
@@ -125,7 +158,7 @@ fnptr gethfn (int fn)
     else if (fn == 7)
         return(hcln);
     else if (fn == 8)
-        return(hcg);
+        return(cg);
     else if (fn == 9)
         return(hsigbin);
     else if (fn == 10)
@@ -143,53 +176,97 @@ fnptr gethfn (int fn)
     else if (fn == 17)
         return(han);
     else if (fn == 18)
-        return(hcumg);
+        return(hcg);
     else (error("unknown or invalid detection function"));
     return(hn);
 }
+*/
+/*--------------------------------------------------------------------*/
+
+fnptr getgfnr (int fn) 
+{
+    if (fn == 0)
+        return(ghnr);
+    else if (fn == 1)
+        return(ghrr);
+    else if (fn == 2)
+        return(gexr);
+    else if (fn == 3)
+        return(ghncr);
+    else if (fn == 4)
+        return(gunr);
+    else if (fn == 5)
+        return(ghfr);
+    else if (fn == 6)
+        return(ganr);
+    else if (fn == 7)
+        return(gclnr);
+    else if (fn == 8)
+        return(gcgr);
+    else if (fn == 9)
+        return(gsigbinr);
+    else if (fn == 10)
+        return(gsigr);
+    else if (fn == 11)
+        return(gsigsphr);
+    else if (fn == 12)
+        return(gsigsphr);
+    else if (fn == 14)
+        return(ghhnr);
+    else if (fn == 15)
+        return(ghhrr);
+    else if (fn == 16)
+        return(ghexr);
+    else if (fn == 17)
+        return(ghanr);
+    else if (fn == 18)
+        return(ghcgr);
+    else (error("unknown or invalid detection function"));
+    return(ghnr);
+}
 /*--------------------------------------------------------------------*/
 
 /* 2016-01-01 experimental for polygon integral */
-fnptr getzfn (int fn) 
+fnptr getzfnr (int fn) 
 {
     if (fn == 0)
-        return(hn);
+        return(zhnr);
     else if (fn == 1)
-        return(hr);
+        return(zhrr);
     else if (fn == 2)
-        return(he);
+        return(zexr);
     else if (fn == 3)
-        return(hnc);
+        return(zhncr);
     else if (fn == 4)
-        return(un);
+        return(zunr);
     else if (fn == 5)
-        return(hf);
+        return(zhfr);
     else if (fn == 6)
-        return(hann);
+        return(zanr);
     else if (fn == 7)
-        return(hcln);
+        return(zclnr);
     else if (fn == 8)
-        return(hcg);
+        return(zcgr);
     else if (fn == 9)
-        return(hsigbin);
+        return(zsigbinr);
     else if (fn == 10)
-        return(hsig);
+        return(zsigr);
     else if (fn == 11)
-        return(hsigsph);
+        return(zsigsphr);
     else if (fn == 12)
-        return(hsigsph);
+        return(zsigsphr);
     else if (fn == 14)
-        return(zhn);
+        return(zhhnr);
     else if (fn == 15)
-        return(zhr);
+        return(zhhrr);
     else if (fn == 16)
-        return(zex);
+        return(zhexr);
     else if (fn == 17)
-        return(zan);
+        return(zhanr);
     else if (fn == 18)
-        return(zcumg);
+        return(zhcgr);
     else (error("unknown or invalid detection function"));
-    return(hn);
+    return(ghnr);
 }
 /*--------------------------------------------------------------------*/
 
@@ -200,7 +277,7 @@ gfnptr getgfn (int fn)
     else if (fn == 1)
         return(ghr);
     else if (fn == 2)
-        return(ghe);
+        return(gex);
     else if (fn == 3)
         return(ghnc);
     else if (fn == 4)
@@ -224,19 +301,63 @@ gfnptr getgfn (int fn)
     else if (fn == 13)
         return(gsigsphSN);
     else if (fn == 14)
-        return(lhn);
+        return(ghhn);
     else if (fn == 15)
-        return(lhr);
+        return(ghhr);
     else if (fn == 16)
-        return(lex);
+        return(ghex);
     else if (fn == 17)
-        return(lan);
+        return(ghan);
     else if (fn == 18)
-        return(lcg);
+        return(ghcg);
     else (error("unknown or invalid detection function"));
     return(ghn);
 }
 
+zfnptr getzfn (int fn) 
+{
+    if (fn == 0)
+        return(zhn);
+    else if (fn == 1)
+        return(zhr);
+    else if (fn == 2)
+        return(zex);
+    else if (fn == 3)
+        return(zhnc);
+    else if (fn == 4)
+        return(zun); /* error("uniform detection function not allowed here"); */
+    else if (fn == 5)
+        return(zhf);
+    else if (fn == 6)
+        return(zan);
+    else if (fn == 7)
+        return(zcln);
+    else if (fn == 8)
+        return(zcg);
+    else if (fn == 9)
+        return(zsigbin);
+    else if (fn == 10)
+        return(zsig);
+    else if (fn == 11)
+        return(zsigsph);
+    else if (fn == 12)
+        return(zsigSN);
+    else if (fn == 13)
+        return(zsigsphSN);
+    else if (fn == 14)
+        return(zhhn);
+    else if (fn == 15)
+        return(zhhr);
+    else if (fn == 16)
+        return(zhex);
+    else if (fn == 17)
+        return(zhan);
+    else if (fn == 18)
+        return(zhcg);
+    else (error("unknown or invalid detection function"));
+    return(zhn);
+}
+
 gfnLptr getgfnL (int fn) 
 {
     if (fn == 0)
@@ -244,7 +365,7 @@ gfnLptr getgfnL (int fn)
     else if (fn == 1)
         return(ghrL);
     else if (fn == 2)
-        return(gheL);
+        return(gexL);
     else if (fn == 3)
         return(ghncL);
     else if (fn == 4)
@@ -268,55 +389,55 @@ gfnLptr getgfnL (int fn)
     else if (fn == 13)
         return(gsigsphSNL);
     else if (fn == 14)
-        return(lhnL);
+        return(ghhnL);
     else if (fn == 15)
-        return(lhrL);
+        return(ghhrL);
     else if (fn == 16)
-        return(lexL);
+        return(ghexL);
     else if (fn == 17)
-        return(lanL);
+        return(ghanL);
     else if (fn == 18)
-        return(lcgL);
+        return(ghcgL);
     else (error("unknown or invalid detection function"));
     return(ghnL);
 }
 
 /*--------------------------------------------------------------------*/
-double hn (double param [], double r) {
-    return(param[0] * exp(- r * r / 2 / param[1] / param[1]));
+double zhnr (double param [], double r) {
+    return(-log(1-param[0] * exp(- r * r / 2 / param[1] / param[1])));
 }
 /*--------------------------------------------------------------------*/
-double hr (double param [], double r) {
-    return(param[0] * (1 - exp(- pow(r / param[1], -param[2]))));
+double zhrr (double param [], double r) {
+    return(-log(1-param[0] * (1 - exp(- pow(r / param[1], -param[2])))));
 }
 /*--------------------------------------------------------------------*/
-double he (double param [], double r) {
-    return (param[0] * exp(-r / param[1]));
+double zexr (double param [], double r) {
+    return (-log(1-param[0] * exp(-r / param[1])));
 }
 /*--------------------------------------------------------------------*/
-double hnc (double param [], double r) {
+double zhncr (double param [], double r) {
     double temp;
     temp = param[0] * exp(- r * r  / 2 / param[1] / param[1]);
     if (round(param[2]) > 1) temp = 1 - pow(1 - temp, param[2]);
-    return (temp);
+    return (-log(1-temp));
 }
 /*--------------------------------------------------------------------*/
-double un (double param [], double r) {
-    if (r<param[1]) return (param[0]);
+double zunr (double param [], double r) {
+    if (r<param[1]) return (-log(1-param[0]));
     else return (0);
 }
 /*--------------------------------------------------------------------*/
-double hf (double param [], double r) {
+double zhfr (double param [], double r) {
     if (r<param[2]) return (param[0]);
-    else return (param[0] * exp(-(r-param[2]) / param[1]));
+    else return (-log(1-param[0] * exp(-(r-param[2]) / param[1])));
 }
 /*--------------------------------------------------------------------*/
-double hann (double param [], double r) {
-    return (param[0] * exp(-(r-param[2])*(r-param[2]) / 2 /
-        param[1] / param[1]));
+double zanr (double param [], double r) {
+    return (-log(1-param[0] * exp(-(r-param[2])*(r-param[2]) / 2 /
+				  param[1] / param[1])));
 }
 /*--------------------------------------------------------------------*/
-double hcln (double param [], double r) {
+double zclnr (double param [], double r) {
     double g0, sigma, z, CV2, meanlog, sdlog;
     g0 = param[0];
     sigma = param[1];
@@ -324,26 +445,26 @@ double hcln (double param [], double r) {
     CV2 = z*z/sigma/sigma;
     meanlog = log(sigma) - log(1 + CV2)/2;
     sdlog = sqrt(log(1 + CV2));
-    return g0 * plnorm(r,meanlog,sdlog,0,0); 
+    return (-log(1-g0 * plnorm(r,meanlog,sdlog,0,0))); 
 }
 /*--------------------------------------------------------------------*/
-double hcg (double param [], double r) {
-    return param[0] * pgamma(r,param[2],param[1]/param[2],0,0); 
+double zcgr (double param [], double r) {
+    return (-log(1-param[0] * pgamma(r,param[2],param[1]/param[2],0,0))); 
 }
 /*--------------------------------------------------------------------*/
 
 /* binary signal strength - (beta0-c)/sdS, beta1/sdS */
-double hsigbin  (double param [], double r) {
+double zsigbinr  (double param [], double r) {
     double gam, b0, b1;
     b0 = param[0];
     b1 = param[1];
     gam = -(b0 + b1 * r);
-    return (pnorm(gam,0,1,0,0));    /* upper */
+    return (-log(1-pnorm(gam,0,1,0,0)));    /* upper */
 }
 /*--------------------------------------------------------------------*/
 
 /* signal strength - beta0, beta1, sdS */
-double hsig (double param [], double r) {
+double zsigr (double param [], double r) {
     double mu, gam;
     double beta0, beta1, sdS, cut;
     beta0 = param[0];
@@ -352,12 +473,12 @@ double hsig (double param [], double r) {
     cut = param[3];
     mu = beta0 + beta1 * r;
     gam = (cut - mu) / sdS;
-    return (pnorm(gam,0,1,0,0));    /* upper */
+    return (-log(1-pnorm(gam,0,1,0,0)));    /* upper */
 }
 /*--------------------------------------------------------------------*/
 
 /* signal strength with spherical spreading - beta0, beta1, sdS */
-double hsigsph (double param [], double r) {
+double zsigsphr (double param [], double r) {
     double mu, gam;
     double beta0, beta1, sdS, cut;
     beta0 = param[0];
@@ -366,68 +487,37 @@ double hsigsph (double param [], double r) {
     cut = param[3];
     mu =  beta0 + beta1 * (r-1) - 10 * log(r*r) / M_LN10;
     gam = (cut - mu) / sdS;
-    return (pnorm(gam,0,1,0,0));    /* upper */
+    return (-log(1-pnorm(gam,0,1,0,0)));    /* upper */
 }
 /*--------------------------------------------------------------------*/
 
 /* hazard halfnormal */
-double hhn (double param [], double r) {
-    /* Rprintf("%6.3f %6.3f %6.3f \n", r, param[0], param[1]); */
-    return(1 - exp( - param[0] * exp(- r * r / 2 / param[1] / param[1])));
-}
-/*--------------------------------------------------------------------*/
-
-/* hazard hazard rate */
-double hhr (double param [], double r) {
-    return(1 - exp( - param[0] * ( 1 - exp(- pow(r / param[1], -param[2])))));
-}
-/*--------------------------------------------------------------------*/
-
-/* hazard exponential */
-double hex (double param [], double r) {
-    return (1 - exp( - param[0] * exp(-r / param[1])));
-}
-/*--------------------------------------------------------------------*/
-
-/* hazard annular normal */
-double han (double param [], double r) {
-    return (1 - exp( - param[0] * exp(-(r-param[2])*(r-param[2]) / 2 /
-				   param[1] / param[1])));
-}
-/*--------------------------------------------------------------------*/
-
-/* hazard cumulative gamma */
-double hcumg (double param [], double r) {
-    return (1 - exp( - (1 - exp( - param[0] * exp(-r / param[1])))));
-}
-
-/* hazard halfnormal */
-double zhn (double param [], double r) {
+double zhhnr (double param [], double r) {
     return(param[0] * exp(- r * r / 2 / param[1] / param[1]));
 }
 /*--------------------------------------------------------------------*/
 
 /* hazard hazard rate */
-double zhr (double param [], double r) {
+double zhhrr (double param [], double r) {
     return(param[0] * ( 1 - exp(- pow(r / param[1], -param[2]))));
 }
 /*--------------------------------------------------------------------*/
 
 /* hazard exponential */
-double zex (double param [], double r) {
+double zhexr (double param [], double r) {
     return (param[0] * exp(-r / param[1]));
 }
 /*--------------------------------------------------------------------*/
 
 /* hazard annular normal */
-double zan (double param [], double r) {
+double zhanr (double param [], double r) {
     return (param[0] * exp(-(r-param[2])*(r-param[2]) / 2 /
 				   param[1] / param[1]));
 }
 /*--------------------------------------------------------------------*/
 
 /* hazard cumulative gamma */
-double zcumg (double param [], double r) {
+double zhcgr (double param [], double r) {
     return ((1 - exp( - param[0] * exp(-r / param[1]))));
 }
 
@@ -467,7 +557,7 @@ double ghr
 /*--------------------------------------------------------------------*/
 
 /* exponential */
-double ghe
+double gex
     (int k, int m, int c, double gsbval[], int cc, double traps[],
     double mask[], int kk, int mm, double miscparm [])
 {
@@ -642,7 +732,7 @@ double gsigsphSN
 /*--------------------------------------------------------------------*/
 
 /* hazard halfnormal */
-double lhn
+double ghhn
     (int k, int m, int c, double gsbval[], int cc, double traps[],
     double mask[], int kk, int mm, double miscparm [])
 {
@@ -659,7 +749,7 @@ double lhn
 /*--------------------------------------------------------------------*/
 
 /* hazard hazard rate */
-double lhr
+double ghhr
     (int k, int m, int c, double gsbval[], int cc, double traps[],
     double mask[], int kk, int mm, double miscparm [])
 {
@@ -673,7 +763,7 @@ double lhr
 /*--------------------------------------------------------------------*/
 
 /* hazard exponential */
-double lex
+double ghex
     (int k, int m, int c, double gsbval[], int cc, double traps[],
     double mask[], int kk, int mm, double miscparm [])
 {
@@ -686,7 +776,7 @@ double lex
 /*--------------------------------------------------------------------*/
 
 /* hazard annular normal */
-double lan
+double ghan
     (int k, int m, int c, double gsbval[], int cc, double traps[],
     double mask[], int kk, int mm, double miscparm [])
 {
@@ -700,7 +790,7 @@ double lan
 /*--------------------------------------------------------------------*/
 
 /* hazard cumulative gamma */
-double lcg
+double ghcg
     (int k, int m, int c, double gsbval[], int cc, double traps[],
     double mask[], int kk, int mm, double miscparm [])
 {
@@ -713,6 +803,289 @@ double lcg
 }
 /*====================================================================*/
 
+/* halfnormal */
+double zhn
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double g;
+    g = gsbval[c] * exp(-d2(k, m, traps, mask, kk, mm) / 2 /
+		    gsbval[cc + c] / gsbval[cc + c]);
+    return (-log(1-g));
+}
+/*--------------------------------------------------------------------*/
+
+/* compound halfnormal */
+double zhnc
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double temp;
+    temp = exp(-d2(k, m, traps, mask, kk, mm) / 2 / gsbval[cc + c] / gsbval[cc + c]);
+    temp = 1 - pow(1 - temp, gsbval[2*cc + c]);
+
+    return (-log(1-gsbval[c] * temp));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard rate */
+double zhr
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double g;
+    g = gsbval[c] * (1 - exp(-
+       pow(sqrt(d2(k,m,traps,mask,kk,mm)) / gsbval[cc + c], - gsbval[cc * 2 + c])));
+    return (-log(1-g));
+}
+/*--------------------------------------------------------------------*/
+
+/* exponential */
+double zex
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double g;
+    g = gsbval[c] * exp(-sqrt(d2(k,m,traps,mask,kk,mm)) / gsbval[cc + c]);
+    return (-log(1-g));
+}
+/*--------------------------------------------------------------------*/
+
+/* uniform */
+double zun
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    if (d2(k, m, traps, mask, kk, mm) < (gsbval[cc + c]*gsbval[cc + c]))
+	return (-log(1-gsbval[c]));
+    else return (0);
+}
+/*--------------------------------------------------------------------*/
+
+/* 'flat-topped exponential' 2009 09 01 */
+double zhf
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, w, g0, sigma;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    g0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    w = gsbval[cc * 2 + c];
+    if (d<w) return (-log(1-g0));
+    else return (-log(1-g0 * exp(-(d-w) / sigma)));
+}
+/*--------------------------------------------------------------------*/
+
+/* annular halfnormal 2010-06-15 */
+double zan
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, w, g0, sigma;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    g0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    w = gsbval[cc * 2 + c];
+    return (-log(1-g0 * exp(-(d-w)*(d-w) / 2 / sigma / sigma)));
+}
+/*--------------------------------------------------------------------*/
+
+/* cumulative lognormal 2010-10-10 */
+double zcln
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, g0, sigma, z, CV2, meanlog, sdlog;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    g0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    z = gsbval[cc * 2 + c];
+    CV2 = z*z/sigma/sigma;
+    meanlog = log(sigma) - log(1 + CV2)/2;
+    sdlog = sqrt(log(1 + CV2));
+    return -log(1-g0 * plnorm(d,meanlog,sdlog,0,0)); 
+}
+/*--------------------------------------------------------------------*/
+
+/* cumulative normal 2010-10-11 */
+double zcn
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, g0, sigma, z;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    g0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    z = gsbval[cc * 2 + c];
+    return -log(1-g0 * pnorm(d,sigma,z,0,0)); 
+}
+/*--------------------------------------------------------------------*/
+
+/* cumulative gamma 2010-10-13 */
+double zcg
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, g0, sigma, z;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    g0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    z = gsbval[cc * 2 + c];
+    return -log(1-g0 * pgamma(d,z,sigma/z,0,0)); 
+}
+/*--------------------------------------------------------------------*/
+
+/* reverse sigmoid */
+double zrs
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, g0, sigma, z, x;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    g0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    z = gsbval[cc * 2 + c];
+    x = z * (d - sigma);
+    return( -log(1-g0 * (1 + (1 - expmin(x)) / (1 + expmin(x))) / 2 )) ;
+}
+/*--------------------------------------------------------------------*/
+
+/* binary signal strength - (beta0-c)/sdS, beta1/sdS */
+double zsigbin
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double gam, b0, b1;
+    b0 = gsbval[c];
+    b1 = gsbval[cc + c];
+    gam = -(b0 + b1 * sqrt(d2(k,m, traps, mask, kk, mm)));
+    return (-log(1-pnorm(gam,0,1,0,0)));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* signal strength - beta0, beta1, sdS */
+double zsig
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm []) {
+    double mu, gam, sdS;
+    mu = mufn (k, m, gsbval[c], gsbval[cc + c], traps, mask, kk, mm, 0);
+    sdS = gsbval[cc * 2 + c];
+    gam = (miscparm[0] - mu) / sdS;
+    return (-log(1-pnorm(gam,0,1,0,0)));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* signal strength with spherical spreading - beta0, beta1, sdS */
+double zsigsph
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm []) {
+    double mu, gam, sdS;
+    mu = mufn (k, m, gsbval[c], gsbval[cc + c], traps, mask, kk, mm, 1);
+    sdS = gsbval[cc * 2 + c];
+    gam = (miscparm[0] - mu) / sdS;
+    return (-log(1-pnorm(gam,0,1,0,0)));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* signal-noise without spherical spreading - beta0, beta1, sdS */
+double zsigSN
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm []) {
+    double cut, muS, sdS, muN, sdN;
+    muS = mufn (k, m, gsbval[c], gsbval[cc + c], traps, mask, kk, mm, 0);
+    sdS = gsbval[cc * 2 + c];
+    cut = miscparm[0];
+    muN = miscparm[1];
+    sdN = miscparm[2];
+    return (-log(1-pnorm(cut, muS-muN, sqrt(sdS*sdS+sdN*sdN), 0, 0)));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* signal-noise with spherical spreading - beta0, beta1, sdS */
+/* interpret sdS as sqrt(var(S-N)) */
+double zsigsphSN
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm []) {
+    double cut, muS, sdS, muN, sdN;
+    muS = mufn (k, m, gsbval[c], gsbval[cc + c], traps, mask, kk, mm, 1);
+    sdS = gsbval[cc * 2 + c];
+    cut = miscparm[0];
+    muN = miscparm[1];    
+    sdN = miscparm[2];    
+    return (-log(1-pnorm(cut, muS-muN, sqrt(sdS*sdS+sdN*sdN), 0, 0)));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard halfnormal */
+double zhhn
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double lambda0, sigma;
+    double invdenom = 1.0;
+    lambda0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    return (lambda0 * invdenom *  exp(-d2(k, m, traps, mask, kk, mm)
+						/ 2 / sigma / sigma));    
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard hazard rate */
+double zhhr
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, lambda0, sigma, z;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    lambda0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    z = gsbval[cc * 2 + c];
+    return (lambda0 * ( 1 - exp(- pow(d /sigma , - z))));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard exponential */
+double zhex
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double lambda0, sigma;
+    lambda0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    return (lambda0 * exp(-sqrt(d2(k,m,traps,mask,kk,mm))
+						/ sigma));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard annular normal */
+double zhan
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, w, lambda0, sigma;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    lambda0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    w = gsbval[cc * 2 + c];
+    return (lambda0 * exp(-(d-w)*(d-w) / 2 / sigma / sigma));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard cumulative gamma */
+double zhcg
+    (int k, int m, int c, double gsbval[], int cc, double traps[],
+    double mask[], int kk, int mm, double miscparm [])
+{
+    double d, lambda0, sigma, z;
+    d = sqrt(d2(k,m,traps,mask,kk,mm));
+    lambda0 = gsbval[c];
+    sigma = gsbval[cc + c];
+    z = gsbval[cc * 2 + c];
+    return (lambda0 * pgamma(d,z,sigma/z,0,0)); 
+}
+/*====================================================================*/
+
 /* halfnormal */
 double ghnL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
@@ -747,7 +1120,7 @@ double ghrL
 /*--------------------------------------------------------------------*/
 
 /* exponential */
-double gheL
+double gexL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
     int kk, double miscparm [])
 {
@@ -922,7 +1295,7 @@ double gsigsphSNL
 /*--------------------------------------------------------------------*/
 
 /* hazard halfnormal */
-double lhnL
+double ghhnL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
     int kk, double miscparm [])
 {
@@ -937,7 +1310,7 @@ double lhnL
 /*--------------------------------------------------------------------*/
 
 /* hazard hazard rate */
-double lhrL
+double ghhrL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
     int kk, double miscparm [])
 {
@@ -953,7 +1326,7 @@ double lhrL
 /*--------------------------------------------------------------------*/
 
 /* hazard exponential */
-double lexL
+double ghexL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
     int kk, double miscparm [])
 {
@@ -968,7 +1341,7 @@ double lexL
 /*--------------------------------------------------------------------*/
 
 /* hazard annular normal */
-double lanL
+double ghanL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
     int kk, double miscparm [])
 {
@@ -984,7 +1357,7 @@ double lanL
 /*--------------------------------------------------------------------*/
 
 /* hazard cumulative gamma */
-double lcgL
+double ghcgL
     (int k, int m, int c, double gsbval[], int cc, double dist2[],
     int kk, double miscparm [])
 {
@@ -1061,4 +1434,125 @@ double mufnL (
 
 }
 /*==============================================================================*/
+/* renamed from hn etc 2017-03-21 */
 
+/*--------------------------------------------------------------------*/
+double ghnr (double param [], double r) {
+    return(param[0] * exp(- r * r / 2 / param[1] / param[1]));
+}
+/*--------------------------------------------------------------------*/
+double ghrr (double param [], double r) {
+    return(param[0] * (1 - exp(- pow(r / param[1], -param[2]))));
+}
+/*--------------------------------------------------------------------*/
+double gexr (double param [], double r) {
+    return (param[0] * exp(-r / param[1]));
+}
+/*--------------------------------------------------------------------*/
+double ghncr (double param [], double r) {
+    double temp;
+    temp = param[0] * exp(- r * r  / 2 / param[1] / param[1]);
+    if (round(param[2]) > 1) temp = 1 - pow(1 - temp, param[2]);
+    return (temp);
+}
+/*--------------------------------------------------------------------*/
+double gunr (double param [], double r) {
+    if (r<param[1]) return (param[0]);
+    else return (0);
+}
+/*--------------------------------------------------------------------*/
+double ghfr (double param [], double r) {
+    if (r<param[2]) return (param[0]);
+    else return (param[0] * exp(-(r-param[2]) / param[1]));
+}
+/*--------------------------------------------------------------------*/
+double ganr (double param [], double r) {
+    return (param[0] * exp(-(r-param[2])*(r-param[2]) / 2 /
+        param[1] / param[1]));
+}
+/*--------------------------------------------------------------------*/
+double gclnr (double param [], double r) {
+    double g0, sigma, z, CV2, meanlog, sdlog;
+    g0 = param[0];
+    sigma = param[1];
+    z = param[2];
+    CV2 = z*z/sigma/sigma;
+    meanlog = log(sigma) - log(1 + CV2)/2;
+    sdlog = sqrt(log(1 + CV2));
+    return g0 * plnorm(r,meanlog,sdlog,0,0); 
+}
+/*--------------------------------------------------------------------*/
+double gcgr (double param [], double r) {
+    return param[0] * pgamma(r,param[2],param[1]/param[2],0,0); 
+}
+/*--------------------------------------------------------------------*/
+
+/* binary signal strength - (beta0-c)/sdS, beta1/sdS */
+double gsigbinr  (double param [], double r) {
+    double gam, b0, b1;
+    b0 = param[0];
+    b1 = param[1];
+    gam = -(b0 + b1 * r);
+    return (pnorm(gam,0,1,0,0));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* signal strength - beta0, beta1, sdS */
+double gsigr (double param [], double r) {
+    double mu, gam;
+    double beta0, beta1, sdS, cut;
+    beta0 = param[0];
+    beta1 = param[1];
+    sdS = param[2];
+    cut = param[3];
+    mu = beta0 + beta1 * r;
+    gam = (cut - mu) / sdS;
+    return (pnorm(gam,0,1,0,0));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* signal strength with spherical spreading - beta0, beta1, sdS */
+double gsigsphr (double param [], double r) {
+    double mu, gam;
+    double beta0, beta1, sdS, cut;
+    beta0 = param[0];
+    beta1 = param[1];
+    sdS = param[2];
+    cut = param[3];
+    mu =  beta0 + beta1 * (r-1) - 10 * log(r*r) / M_LN10;
+    gam = (cut - mu) / sdS;
+    return (pnorm(gam,0,1,0,0));    /* upper */
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard halfnormal */
+double ghhnr (double param [], double r) {
+    /* Rprintf("%6.3f %6.3f %6.3f \n", r, param[0], param[1]); */
+    return(1 - exp( - param[0] * exp(- r * r / 2 / param[1] / param[1])));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard hazard rate */
+double ghhrr (double param [], double r) {
+    return(1 - exp( - param[0] * ( 1 - exp(- pow(r / param[1], -param[2])))));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard exponential */
+double ghexr (double param [], double r) {
+    return (1 - exp( - param[0] * exp(-r / param[1])));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard annular normal */
+double ghanr (double param [], double r) {
+    return (1 - exp( - param[0] * exp(-(r-param[2])*(r-param[2]) / 2 /
+				   param[1] / param[1])));
+}
+/*--------------------------------------------------------------------*/
+
+/* hazard cumulative gamma */
+double ghcgr (double param [], double r) {
+    return (1 - exp( - (1 - exp( - param[0] * exp(-r / param[1])))));
+}
+/*--------------------------------------------------------------------*/
